#!/usr/bin/env ruby

require "sinatra/base" # web
require "sinatra/cookies"
require "slim"
require "sinatra/namespace" # api
require "doc_my_routes" # api doc
require "bcrypt" # security
require "rack/csrf"
require "securerandom"
require "sequel" # db
require "json" # helpers
require "active_support"
require "active_support/core_ext"
require "pry" # debug
require "dotenv" # env variables
require "colorize"

# Load environment variables
module Upupup
  def self.get_env
    ENV.fetch("UPUPUP_ENV") { "development" }
  end

  APP_ENV = get_env
  Dotenv.load(".env.#{get_env}.local", ".env.local", ".env")
end

# Setup module variables
module Upupup
  DB = Sequel.connect ENV["UPUPUP_DB"]
  BASE_URL = ENV["UPUPUP_BASE_URL"] || "http://localhost:4567"
  SECRET = ENV["UPUPUP_SECRET"]
  # CODE_VERSION = DB[:meta].first[:code_version]
  # CODE_DATE = DB[:meta].first[:code_date]

  module Utils
    module Url
      # usage: base_url('base', 'to', '2', raw: false) => "http://..../base/to/2"
      # @param [Boolean] raw set to false by default for a string output, if true it will be an URI
      def base_url(*paths, raw: false)
        uri = URI.join(Upupup::BASE_URL, *(paths.map(&:to_s)))
        return uri if raw
        return uri.to_s
      end
    end
  end

  include Utils::Url

  module Systems
  end
end

Dir[File.join(__dir__, 'utils', '*.rb')].each { |file| require file }
Dir[File.join(__dir__, 'models', '*.rb')].each { |file| require file }
require_relative "./worker"

module Upupup
  WorkerInstance = Upupup::Worker.new
end

Upupup::WorkerInstance.start_thread_loops # join to block

# Then we load all the systems
Dir[File.join(__dir__, 'systems', '*.rb')].each { |file| require file }

# Static file serving in this file because it is overkill to create a file for this
class Upupup::Systems::PublicSystem < Sinatra::Base
  set :public_folder, "public"
end

# Main app
class Upupup::App < Sinatra::Base
  DocMyRoutes.configure do |config|
    config.title = "Upupup"
    config.description = "Upupup JSON REST API documentation"
  end

  set :session_secret, Upupup::SECRET
  enable :sessions
  set :protection, :except => :json_csrf if Upupup::APP_ENV != "test"

  Upupup::Systems.constants
    .filter { |system| system.to_s =~ /System$/ }
    .each { |system|
    use Upupup::Systems.const_get(system)
      puts "Loaded #{system.to_s.green}"
    }

  include JSON::API
  not_found do
    if accept_json?
      { message: "No route or entity \"#{request.path}\" found" }.to_json
    else
      raise
    end
  end

  get "/api/meta/v1", provides: "json" do
    Upupup::Systems::ApiList.get_all_api_routes.to_json
  end

  set :bind, ENV["UPUPUP_BIND"] || "127.0.0.1"

  set :environment, Upupup::APP_ENV
  ENV["RACK_ENV"] = Upupup::APP_ENV

  run! if app_file == $0
end
